import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;

public final class BattleCanvas extends Canvas implements CommandListener {
  private DevilWorldMidlet owner;
  private Hero hero;
  private Enemy enemy;
  private String[] message;
  private int status;
  private static final int STOP = 0;
  private static final int OK = 1;
  private static final int TOFIGHT = 2;
  private static final int TOESCAPE = 3;
  private static final int ENEMYTURN = 4;
  private static final int WIN = 5;
  private static final int LOSE = 6;
  private static final int ESCAPE = 7;

  //RXgN^
  BattleCanvas(DevilWorldMidlet owner, Hero hero) {
    this.owner = owner;
    this.hero = hero;
    this.message = new String[4];
    for(int i=0; i<4; i++){ message[i] = ""; }
    setCommandListener(this);
    status = STOP;
  }

  //`
  public void paint(Graphics g) {
    int dw = getWidth();
    int dh = getHeight();
    g.setColor(0,0,0);
    g.fillRect(0,0,dw,dh);

    int fh = g.getFont().getHeight();
    int rh = fh * 5;
    int rt = dh - rh - 1;

    g.drawRegion(owner.enemyimage, (enemy.getImage()-'m')*128, 0, 128, 128, Sprite.TRANS_NONE, dw/2, rt/2, g.HCENTER|g.VCENTER);

    int rl = 8;
    int rr = dw/3 - 8;
    g.setColor(255,255,255);
    g.drawRoundRect(0, rt, dw/3, rh, 8, 8);
    g.drawString("HP:", rl, rt, g.LEFT|g.TOP);
    g.drawString(Integer.toString(hero.getHP()), rr, rt, g.RIGHT|g.TOP);
    g.drawString("LV:", rl, rt+fh, g.LEFT|g.TOP);
    g.drawString(Integer.toString(hero.getLevel()), rr, rt+fh, g.RIGHT|g.TOP);
    g.drawString("EXP:", rl, rt+fh*2, g.LEFT|g.TOP);
    g.drawString(Integer.toString(hero.getEXP()), rr, rt+fh*2, g.RIGHT|g.TOP);
    g.drawString(hero.getWeaponName1(), (rl+rr)/2, rt+fh*3, g.HCENTER|g.TOP);
    g.drawString(hero.getWeaponName2(), (rl+rr)/2, rt+fh*4, g.HCENTER|g.TOP);

    rl = dw/3 + 8;
    rr = dw - 8;
    g.setColor(255,255,255);
    g.drawRoundRect(dw/3, rt, dw*2/3-1, rh, 8, 8);
    for(int i=0; i<4; i++){
      g.drawString(message[i], rl, rt+fh*i, g.LEFT|g.TOP);
    }
    if(status == OK || status == ENEMYTURN || status == WIN || status == LOSE || status == ESCAPE){
      g.setColor(0,255,255);
      g.drawString("OK", (rl+rr)/2, rt+fh*4, g.HCENTER|g.TOP);
    }else if(status == TOFIGHT || status ==TOESCAPE){
      if(status == TOFIGHT){ g.setColor(0,255,255); }else{ g.setColor(192,192,192); }
      g.drawString("", rl, rt+fh*4, g.LEFT|g.TOP);
      if(status == TOESCAPE){ g.setColor(0,255,255); }else{ g.setColor(192,192,192); }
      g.drawString("ɂ", (rl+rr)/2, rt+fh*4, g.LEFT|g.TOP);
    }
  }

  public void startBattle(Enemy enemy){
    this.enemy = enemy;
    message[0] = enemy.getName().concat(" ");
    message[1] = "ꂽ";
    message[2] = "";
    message[3] = "";
    status = TOFIGHT;
  }
  
  public void fight(){
    int attack = hero.getAP() - enemy.getDP();
    if(attack < 1){ attack = 1; }
    enemy.damage(attack);
    message[0] = "";
    message[1] = Integer.toString(attack).concat(" ̃_[W");
    if(enemy.getHP() <= 0){
      message[2] = "";
      message[3] = Integer.toString(enemy.getEXP()).concat(" ̂");
      if(hero.addExp(enemy.getEXP())){
        status = WIN;
      }else{
        status = OK;
      }
    }else{
      status = ENEMYTURN;
    }
  }
  
  public void escape(){
    boolean ok = ((owner.random.nextInt() % 2) == 0);
    message[0] = "ɂ";
    if(ok){
      message[1] = "";
      status = ESCAPE;
    }else{
      message[1] = "ς";
      status = ENEMYTURN;
    }
    message[2] = "";
  }
  
  public void enemyTurn(){
    int attack = enemy.getAP() - hero.getDP();
    if(attack < 1){ attack = 1; }
    hero.damage(attack);
    message[0] = "ꂽ";
    message[1] = Integer.toString(attack).concat(" ̃_[W");
    if(hero.getHP() <= 0){
      message[2] = "";
      status = LOSE;
    }else{
      message[2] = "";
      status = TOFIGHT;
    }
  }
  
  public void win(){
    message[0] = "x  ";
    message[1] = "";
    message[2] = "";
    message[3] = "";
    status = OK;
  }
  
  //L[vXCxg
  public void keyPressed(int keyCode) {
    int action=getGameAction(keyCode);
    if(status == TOFIGHT){
      if (action==LEFT || action==RIGHT){ status = TOESCAPE; }
      if (action==FIRE){ fight(); }
    }else if(status == TOESCAPE){
      if (action==LEFT || action==RIGHT){ status = TOFIGHT; }
      if (action==FIRE){ escape(); }
    }else if(status == ENEMYTURN){
      if (action==FIRE){ enemyTurn(); }
    }else if(status == WIN){
      if (action==FIRE){ win(); }
    }else if(status == LOSE){
      if (action==FIRE){ owner.init(); }
    }else if(status == ESCAPE){
      if (action==FIRE){ owner.escape(); }
    }else if(status == OK){
      if (action==FIRE){ owner.goField(); }
    }
    repaint();
  }

  //R}hCxg
  public void commandAction(Command c,Displayable s) {
  }
}
