import java.io.*;
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;

public final class FieldCanvas extends Canvas implements CommandListener {
  private DevilWorldMidlet owner;
  private int x;
  private int y;
  private char[][] map;
  private String mapname;
  private String[] message;
  private int eventcontinue;
  private int status;
  private static final int sz = 32;
  private static final int WALK = 0;
  private static final int MESSAGE = 1;

  public FieldCanvas(DevilWorldMidlet owner) {
    this.owner = owner;
  }

  public void jump(String mapname, int x, int y) {
    this.x = x;
    this.y = y;
    this.mapname = mapname;
    this.status = WALK;
    try{
      read(mapname);
    }catch(IOException e){}
  }

  public void setMessage(String[] message, int cont){
    this.message = message;
    this.eventcontinue = cont;
    this.status = MESSAGE;
  }

  private void read(String mapname) throws IOException {
    InputStream stream = getClass().getResourceAsStream(mapname);
    int w = 0;
    int h = 0;
    while(true){
      int ch = stream.read();
      if(ch < 0 || ch == ',' || ch == '\r' || ch == '\n'){ break; }
      w = (10 * w) + (ch - '0');
    }
    while(true){
      int ch = stream.read();
      if(ch < 0 || ch == ',' || ch == '\r' || ch == '\n'){ break; }
      h = (10 * h) + (ch - '0');
    }
    map = new char[h][];
    for(int i=0; i<h; i++){
      map[i] = new char[w];
    }
    int y = 0;
    int x = 0;
    while(true){
      int ch = stream.read();
      if(ch < 0){ break; }
      if(ch == '\r' || ch == '\n'){
        if(x > 0){ y++; x=0; }
      }else{
        if(y < h && x < w){ map[y][x] = (char)ch; }
        x++;
      }
    }
  }
  
  public char getMapAt(int x, int y){
    if(x < 0 || y < 0 || y >= map.length || x >= map[y].length){
      return ' ';
    }
    return map[y][x];
  }

  public void setMapAt(int x, int y, char ch){
    if(x < 0 || y < 0 || y >= map.length || x >= map[y].length){
      return;
    }
    map[y][x] = ch;
  }

  public String getMapName(){ return mapname; }

  public void paint(Graphics g) {
    int dw = getWidth();
    int dh = getHeight();
    int ox = dw/2-sz/2;
    int oy = dh/2-sz/2;
    for(int iy=((0-oy)/sz-1); iy < ((dh-oy)/sz+1); iy++){
      for(int ix=((0-ox)/sz-1); ix < ((dw-ox)/sz+1); ix++){
        char pt = getMapAt(x + ix, y + iy);
        if(pt == ' ' || pt == '\0'){
          g.fillRect((ix*sz)+ox, (iy*sz)+oy, sz, sz);
        }else if(pt >= 'a' && pt <= 'z'){
          g.drawRegion(owner.image, (pt-'a')*sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }else if(pt >= 'A' && pt <= 'Z'){
          g.drawRegion(owner.image, (pt-'A')*sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }else if(pt >= '1' && pt <= '9'){
          g.drawRegion(owner.image, sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }else if(pt == '%'){
          g.drawRegion(owner.image, 5*sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }else if(pt == '#' || pt == '&'){
          g.drawRegion(owner.image, 10*sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }else if(pt == '$' || pt == '\\'){
          g.drawRegion(owner.image, 11*sz, 0, sz, sz, Sprite.TRANS_NONE, (ix*sz)+ox, (iy*sz)+oy, g.LEFT|g.TOP);
        }
      }
    }
    g.setColor(0,0,255);
    g.drawRegion(owner.image, 448, 0, sz, sz, Sprite.TRANS_NONE, ox, oy, g.LEFT|g.TOP);
    
    if(status == MESSAGE){
      int fh = g.getFont().getHeight();
      int rh = fh * 5;
      int rt = dh - rh - 1;

      Hero hero = owner.hero;
      int rl = 8;
      int rr = dw/3 - 8;
      g.setColor(0,0,0);
      g.fillRoundRect(0, rt, dw/3, rh, 8, 8);
      g.setColor(255,255,255);
      g.drawRoundRect(0, rt, dw/3, rh, 8, 8);
      g.drawString("HP:", rl, rt, g.LEFT|g.TOP);
      g.drawString(Integer.toString(hero.getHP()), rr, rt, g.RIGHT|g.TOP);
      g.drawString("LV:", rl, rt+fh, g.LEFT|g.TOP);
      g.drawString(Integer.toString(hero.getLevel()), rr, rt+fh, g.RIGHT|g.TOP);
      g.drawString("EXP:", rl, rt+fh*2, g.LEFT|g.TOP);
      g.drawString(Integer.toString(hero.getEXP()), rr, rt+fh*2, g.RIGHT|g.TOP);
      g.drawString(hero.getWeaponName1(), (rl+rr)/2, rt+fh*3, g.HCENTER|g.TOP);
      g.drawString(hero.getWeaponName2(), (rl+rr)/2, rt+fh*4, g.HCENTER|g.TOP);

      rl = dw/3 + 8;
      rr = dw - 8;
      g.setColor(0,0,0);
      g.fillRoundRect(dw/3, rt, dw*2/3-1, rh, 8, 8);
      g.setColor(255,255,255);
      g.drawRoundRect(dw/3, rt, dw*2/3-1, rh, 8, 8);
      for(int i=0; i<message.length; i++){
        g.drawString(message[i], rl, rt+fh*i, g.LEFT|g.TOP);
      }
      g.setColor(0,255,255);
      g.drawString("OK", (rl+rr)/2, rt+fh*4, g.HCENTER|g.TOP);
    }
  }

  public void keyPressed(int keyCode) {
    int action=getGameAction(keyCode);
    if(status == WALK){
      int lx = x;
      int ly = y;
      if(action==LEFT){ x--; }
      if(action==RIGHT){ x++; }
      if(action==UP){ y--; }
      if(action==DOWN){ y++; }
      char ch = getMapAt(x,y);
      if(ch < 'a' || ch > 'g'){ x = lx; y = ly; }
      if(ch < 'a' || ch > 'z'){ Event.exec(owner, this, ch); }
      if(ch >= 'a' && ch <= 'g' && (x != lx || y != ly)){ encount(); }
    }else if(status == MESSAGE){
      if(action==FIRE || eventcontinue==0){ 
        status = WALK;
        if(eventcontinue > 0){
          Event.exec(owner, this, eventcontinue);
        }
      }
    }
    repaint();
  }

  private void encount(){
    int rnd = (owner.random.nextInt() & 0xff);
    if(mapname.equals("/Map1.txt")){
      if(rnd < 13){
        owner.goBattle(new Enemy("fWF",'m',10,3,1,1));
      }else if(rnd < 26){
        owner.goBattle(new Enemy("aWF",'n',15,3,3,2));
      }
    }else if(mapname.equals("/Cave.txt")){
      if(rnd < 26){
        owner.goBattle(new Enemy("qWF",'o',20,7,8,4));
      }else if(rnd < 52){
        owner.goBattle(new Enemy("fhS",'p',30,11,10,5));
      }
    }else if(mapname.equals("/Tower1.txt")){
      if(rnd < 26){
        owner.goBattle(new Enemy("fhS",'p',30,11,10,5));
      }else if(rnd < 52){
        owner.goBattle(new Enemy("ahS",'q',35,20,20,10));
      }
    }else if(mapname.equals("/Tower2.txt")){
      if(rnd < 26){
        owner.goBattle(new Enemy("ahS",'q',35,20,20,10));
      }else if(rnd < 52){
        owner.goBattle(new Enemy("qhS",'r',50,25,26,20));
      }
    }else if(mapname.equals("/Tower3.txt")){
      if(rnd < 52){
        owner.goBattle(new Enemy("qhS",'r',50,25,26,20));
      }
    }
  }
  
  public void commandAction(Command c,Displayable s) {
  }
}
